@@ -17,8 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _ctlib_h_
-#define _ctlib_h_
+#ifndef _tdsguard_gH4tOYBTzynx4DgqFzOEbB_
+#define _tdsguard_gH4tOYBTzynx4DgqFzOEbB_
 
 #include <freetds/tds.h>
 #include <freetds/convert.h>
@@ -47,7 +47,8 @@ extern "C"
  */
 struct _cs_config
 {
-	short cs_expose_formats;
+	bool cs_expose_formats;
+	bool cs_note_empty_data;
 };
 
 /* Code changed for error handling */
@@ -148,9 +149,10 @@ struct _cs_context
 	/* code changes ends here - CT_DIAG - 02 */
 
 	struct cs_diag_msg *msgstore;
-	CS_CSLIBMSG_FUNC _cslibmsg_cb;
-	CS_CLIENTMSG_FUNC _clientmsg_cb;
-	CS_SERVERMSG_FUNC _servermsg_cb;
+	CS_CSLIBMSG_FUNC cslibmsg_cb;
+	CS_CLIENTMSG_FUNC clientmsg_cb;
+	CS_SERVERMSG_FUNC servermsg_cb;
+	CS_INTERRUPT_FUNC interrupt_cb;
 	/* code changes start here - CS_CONFIG - 01*/
 	void *userdata;
 	int userdata_len;
@@ -187,8 +189,9 @@ struct _cs_connection
 	CS_CONTEXT *ctx;
 	TDSLOGIN *tds_login;
 	TDSSOCKET *tds_socket;
-	CS_CLIENTMSG_FUNC _clientmsg_cb;
-	CS_SERVERMSG_FUNC _servermsg_cb;
+	CS_CLIENTMSG_FUNC clientmsg_cb;
+	CS_SERVERMSG_FUNC servermsg_cb;
+	CS_INTERRUPT_FUNC interrupt_cb;
 	void *userdata;
 	int userdata_len;
 	CS_LOCALE *locale;
@@ -379,20 +382,23 @@ typedef union
  */
 TDSRET _ct_handle_server_message(const TDSCONTEXT * ctxptr, TDSSOCKET * tdsptr, TDSMESSAGE * msgptr);
 int _ct_handle_client_message(const TDSCONTEXT * ctxptr, TDSSOCKET * tdsptr, TDSMESSAGE * msgptr);
+int _ct_handle_interrupt(void * ptr);
 TDS_SERVER_TYPE _ct_get_server_type(TDSSOCKET *tds, int datatype);
 int _ct_bind_data(CS_CONTEXT *ctx, TDSRESULTINFO * resinfo, TDSRESULTINFO *bindinfo, CS_INT offset);
 int _ct_get_client_type(const TDSCOLUMN *col, bool describe);
-void _ctclient_msg(CS_CONNECTION * con, const char *funcname, int layer, int origin, int severity, int number,
+void _ctclient_msg(CS_CONTEXT *ctx, CS_CONNECTION * con, const char *funcname,
+		   int layer, int origin, int severity, int number,
 		   const char *fmt, ...);
 CS_INT _ct_diag_clearmsg(CS_CONTEXT * context, CS_INT type);
 void _cs_locale_free(CS_LOCALE *locale);
 CS_LOCALE *_cs_locale_copy(CS_LOCALE *orig);
 int _cs_locale_copy_inplace(CS_LOCALE *new_locale, CS_LOCALE *orig);
+CS_INT _ct_get_string_length(const char *buf, CS_INT buflen);
 
 int _cs_convert_not_client(CS_CONTEXT *ctx, const TDSCOLUMN *curcol, CONV_RESULT *convert_buffer, unsigned char **p_src);
 
 CS_RETCODE _cs_convert(CS_CONTEXT * ctx, const CS_DATAFMT_COMMON * srcfmt, CS_VOID * srcdata,
-	const CS_DATAFMT_COMMON * destfmt, CS_VOID * destdata, CS_INT * resultlen);
+	const CS_DATAFMT_COMMON * destfmt, CS_VOID * destdata, CS_INT * resultlen, TDS_SERVER_TYPE desttype);
 bool _ct_is_large_identifiers_version(CS_INT version);
 const CS_DATAFMT_COMMON * _ct_datafmt_common(CS_CONTEXT * ctx, const CS_DATAFMT * datafmt);
 const CS_DATAFMT_LARGE *_ct_datafmt_conv_in(CS_CONTEXT * ctx, const CS_DATAFMT * datafmt, CS_DATAFMT_LARGE * fmtbuf);
