@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _sql_h_
-#define _sql_h_
+#ifndef _tdsguard_gQHaPEvp2dAAa45TAutROb_
+#define _tdsguard_gQHaPEvp2dAAa45TAutROb_
 
 #define TDS_DONT_DEFINE_DEFAULT_FUNCTIONS
 #include <freetds/tds.h>
@@ -74,7 +74,9 @@ extern "C"
 #endif
 
 #include <freetds/pushvis.h>
-#if defined(__GNUC__) && __GNUC__ >= 4 && !defined(__MINGW32__)
+#ifdef __clang__
+#define ODBC_API SQL_API __attribute__((visibility("default")))
+#elif defined(__GNUC__) && __GNUC__ >= 4 && !defined(__MINGW32__)
 #define ODBC_API SQL_API __attribute__((externally_visible))
 #else
 #define ODBC_API SQL_API
@@ -100,6 +102,7 @@ struct _sql_error
 	int linenum;
 	int msgstate;
 	int row;
+	bool msg_is_static;
 };
 
 struct _sql_errors
@@ -115,7 +118,7 @@ typedef struct _sql_errors TDS_ERRS;
 #if ENABLE_EXTRA_CHECKS
 void odbc_check_struct_extra(void *p);
 #else
-static inline void odbc_check_struct_extra(void *p) {}
+static inline void odbc_check_struct_extra(void *p TDS_UNUSED) {}
 #endif
 
 #define ODBC_RETURN(handle, rc) \
@@ -412,6 +415,8 @@ struct _hstmt
 	 */
 	unsigned need_reprepare:1;
 	unsigned param_data_called:1;
+	unsigned params_queried:1;
+	unsigned params_set:1;
 	/* end prepared query stuff */
 
 	/** parameters saved */
@@ -524,7 +529,9 @@ bool get_login_info(HWND hwndParent, TDSLOGIN * login);
 	ODBC_PARAM(ServerSPN) \
 	ODBC_PARAM(AttachDbFilename) \
 	ODBC_PARAM(ApplicationIntent) \
-	ODBC_PARAM(Timeout)
+	ODBC_PARAM(Timeout) \
+	ODBC_PARAM(Encrypt) \
+	ODBC_PARAM(HostNameInCertificate)
 
 #define ODBC_PARAM(p) ODBC_PARAM_##p,
 enum {
@@ -533,6 +540,9 @@ enum {
 };
 #undef ODBC_PARAM
 
+#define ODBC_PARAM(p) extern const char odbc_param_##p[];
+ODBC_PARAM_LIST
+#undef ODBC_PARAM
 
 /*
  * connectparams.h
@@ -547,11 +557,12 @@ typedef struct {
  * Parses a connection string for SQLDriverConnect().
  * \param connect_string      point to connection string
  * \param connect_string_end  point to end of connection string
- * \param connection          structure where to store informations
+ * \param connection          structure where to store information
  * \return true if error, false otherwise
  */
-bool odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char *connect_string_end, TDSLOGIN * login, TDS_PARSED_PARAM *parsed_params);
-int odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login);
+bool odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char *connect_string_end,
+			       TDSLOGIN * login, TDS_PARSED_PARAM *parsed_params);
+bool odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login);
 #ifdef _WIN32
 int odbc_build_connect_string(TDS_ERRS *errs, TDS_PARSED_PARAM *params, char **out);
 #endif
@@ -559,7 +570,8 @@ int odbc_build_connect_string(TDS_ERRS *errs, TDS_PARSED_PARAM *params, char **o
 /*
  * convert_tds2sql.c
  */
-SQLLEN odbc_tds2sql_col(TDS_STMT * stmt, TDSCOLUMN *curcol, int desttype, TDS_CHAR * dest, SQLULEN destlen, const struct _drecord *drec_ixd);
+SQLLEN odbc_tds2sql_col(TDS_STMT * stmt, TDSCOLUMN *curcol, int desttype,
+			TDS_CHAR * dest, SQLULEN destlen, const struct _drecord *drec_ixd);
 SQLLEN odbc_tds2sql_int4(TDS_STMT * stmt, TDS_INT *src, int desttype, TDS_CHAR * dest, SQLULEN destlen);
 
 
@@ -586,7 +598,7 @@ void tvp_free(SQLTVP *tvp);
  * odbc.c
  */
 
-SQLRETURN _SQLRowCount(SQLHSTMT hstmt, SQLLEN FAR * pcrow);
+SQLRETURN odbc_SQLRowCount(SQLHSTMT hstmt, SQLLEN FAR * pcrow);
 
 /*
  * odbc_checks.h
@@ -631,7 +643,7 @@ typedef union {
 # define _WIDE
 # define ODBC_CHAR SQLCHAR
 #endif
-int odbc_set_stmt_query(struct _hstmt *stmt, const ODBC_CHAR *sql, int sql_len _WIDE);
+SQLRETURN odbc_set_stmt_query(struct _hstmt *stmt, const ODBC_CHAR *sql, int sql_len _WIDE);
 void odbc_set_return_status(struct _hstmt *stmt, unsigned int n_row);
 void odbc_set_return_params(struct _hstmt *stmt, unsigned int n_row);
 
@@ -643,7 +655,8 @@ TDS_SERVER_TYPE odbc_c_to_server_type(int c_type);
 
 unsigned int odbc_get_string_size(int size, const ODBC_CHAR * str _WIDE);
 void odbc_rdbms_version(TDSSOCKET * tds_socket, char *pversion_string);
-SQLINTEGER odbc_get_param_len(const struct _drecord *drec_axd, const struct _drecord *drec_ixd, const TDS_DESC* axd, unsigned int n_row);
+SQLINTEGER odbc_get_param_len(const struct _drecord *drec_axd, const struct _drecord *drec_ixd,
+			      const TDS_DESC* axd, unsigned int n_row);
 
 #ifdef ENABLE_ODBC_WIDE
 DSTR* odbc_dstr_copy_flag(TDS_DBC *dbc, DSTR *s, int size, const ODBC_CHAR * str, int flag);
@@ -657,10 +670,12 @@ DSTR* odbc_dstr_copy(TDS_DBC *dbc, DSTR *s, int size, const ODBC_CHAR * str);
 #endif
 
 
-SQLRETURN odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void FAR * pcbBuffer, const char *s, int len, int flag);
+SQLRETURN odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void FAR * pcbBuffer,
+			       const char *s, int len, int flag);
 #ifdef ENABLE_ODBC_WIDE
 #define odbc_set_string(dbc, buf, buf_len, out_len, s, s_len) \
-	odbc_set_string_flag(dbc, sizeof((buf)->mb) ? (buf) : (buf), buf_len, out_len, s, s_len, (wide) | (sizeof(*(out_len)) == sizeof(SQLSMALLINT)?0:0x10))
+	odbc_set_string_flag(dbc, sizeof((buf)->mb) ? (buf) : (buf), buf_len, out_len, s, s_len, \
+			     (wide) | (sizeof(*(out_len)) == sizeof(SQLSMALLINT)?0:0x10))
 #define odbc_set_string_oct(dbc, buf, buf_len, out_len, s, s_len) \
 	odbc_set_string_flag(dbc, buf, buf_len, out_len, s, s_len, (wide) | (sizeof(*(out_len)) == sizeof(SQLSMALLINT)?0x20:0x30))
 #else
@@ -670,7 +685,8 @@ SQLRETURN odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuf
 	odbc_set_string_flag(dbc, buf, buf_len, out_len, s, s_len, (sizeof(*(out_len)) == sizeof(SQLSMALLINT)?0x20:0x30))
 #endif
 
-#define odbc_set_dstr_oct(dbc, buf, buf_len, out_len, s) odbc_set_string_oct(dbc, buf, buf_len, out_len, tds_dstr_cstr(s), tds_dstr_len(s))
+#define odbc_set_dstr_oct(dbc, buf, buf_len, out_len, s) \
+	odbc_set_string_oct(dbc, buf, buf_len, out_len, tds_dstr_cstr(s), tds_dstr_len(s))
 #define odbc_set_dstr(dbc, buf, buf_len, out_len, s) odbc_set_string(dbc, buf, buf_len, out_len, tds_dstr_cstr(s), tds_dstr_len(s))
 
 SQLSMALLINT odbc_get_concise_sql_type(SQLSMALLINT type, SQLSMALLINT interval);
@@ -695,7 +711,8 @@ const char *odbc_skip_rpc_name(const char *s);
 /*
  * sql2tds.c
  */
-SQLRETURN odbc_sql2tds(TDS_STMT * stmt, const struct _drecord *drec_ixd, const struct _drecord *drec_axd, TDSCOLUMN *curcol, bool compute_row, const TDS_DESC* axd, unsigned int n_row);
+SQLRETURN odbc_sql2tds(TDS_STMT * stmt, const struct _drecord *drec_ixd, const struct _drecord *drec_axd, TDSCOLUMN *curcol,
+		       bool compute_row, const TDS_DESC* axd, unsigned int n_row);
 TDS_INT convert_datetime2server(int bindtype, const void *src, TDS_DATETIMEALL * dta);
 
 /*
@@ -708,7 +725,8 @@ void odbc_bcp_colptr(TDS_DBC *dbc, const void * colptr, int table_column);
 void odbc_bcp_sendrow(TDS_DBC *dbc);
 int odbc_bcp_batch(TDS_DBC *dbc);
 int odbc_bcp_done(TDS_DBC *dbc);
-void odbc_bcp_bind(TDS_DBC *dbc, const void * varaddr, int prefixlen, int varlen, const void * terminator, int termlen, int vartype, int table_column);
+void odbc_bcp_bind(TDS_DBC *dbc, const void * varaddr, int prefixlen, int varlen, const void * terminator, int termlen,
+		   int vartype, int table_column);
 
 /*
  * sqlwchar.c
@@ -735,6 +753,25 @@ void sqlwstr_free(SQLWSTRBUF *bufs);
 
 int odbc_get_wide_canonic(TDSCONNECTION *conn);
 
+/* compatibility with old BCP implementation */
+#define BCPHINTS_OLD 6
+#define SQL_COPT_TDSODBC_IMPL_BCP_CONTROL_OLD	(SQL_COPT_TDSODBC_IMPL_BASE+1)
+
+/* iODBC extensions used by our driver */
+#ifndef SQL_ATTR_DRIVER_UNICODE_TYPE
+#define SQL_ATTR_DRIVER_UNICODE_TYPE 1065
+#endif
+
+#ifndef SQL_DM_CP_UTF16
+#define SQL_DM_CP_UTF16 1
+#endif
+#ifndef SQL_DM_CP_UTF8
+#define SQL_DM_CP_UTF8 2
+#endif
+#ifndef SQL_DM_CP_UCS4
+#define SQL_DM_CP_UCS4 3
+#endif
+
 #include <freetds/popvis.h>
 
 #ifdef __cplusplus
